//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "SignalCfg.h"
#include "Signal.h"
#include "Utils.h"
#include <SysUtils.hpp>

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmSignalConfig *frmSignalConfig = NULL;
//---------------------------------------------------------------------------
__fastcall TfrmSignalConfig::TfrmSignalConfig(TComponent* Owner, Signal* signal, int signal_cnt)
	: TForm(Owner), signal(signal), signal_cnt(signal_cnt)
{
	std::string sWaveType;
	for (int i=0; i<Signal::WAVE_LIMITER; i++)
	{
		if (Signal::GetTypeDescription(i, sWaveType) == 0)
			cbWaveType->Items->Add(sWaveType.c_str());
		else
			cbWaveType->Items->Add("");
	}
	cbChannel->Clear();
	for (int i=0; i<signal_cnt; i++)
	{
        cbChannel->Items->Add(Utils::GetChannelName(i));
	}
	cbChannel->ItemIndex = 0;
	cbChannelChange(NULL);
}

int TfrmSignalConfig::CurrentChannel(void)
{
	return cbChannel->ItemIndex;
}

//---------------------------------------------------------------------------
void __fastcall TfrmSignalConfig::FormShow(TObject *Sender)
{
	this->lvWaves->Items->Count = signal[CurrentChannel()].shape.size();
	lvWaves->Invalidate();
}
//---------------------------------------------------------------------------
void __fastcall TfrmSignalConfig::lvWavesData(TObject *Sender, TListItem *Item)
{
	int id = Item->Index;
	std::string sWaveType;
	struct Signal::S_WAVE wave = signal[CurrentChannel()].shape[id];
	Item->Caption = id + 1;
	if (Signal::GetTypeDescription(wave.shape, sWaveType) == 0)
		Item->SubItems->Add(sWaveType.c_str());
	else
		Item->SubItems->Add("");
	if (wave.shape != Signal::WAVE_NONE)
	{
		Item->SubItems->Add(AnsiString::FloatToStrF(wave.fAmplitude, AnsiString::sffGeneral, 2, 3));
		Item->SubItems->Add(AnsiString::FloatToStrF(wave.fFrequency, AnsiString::sffGeneral, 2, 3));
		Item->SubItems->Add(AnsiString::FloatToStrF(wave.fPhase, AnsiString::sffGeneral, 2, 3));
	}
}
//---------------------------------------------------------------------------
void __fastcall TfrmSignalConfig::lvWavesSelectItem(TObject *Sender,
      TListItem *Item, bool Selected)
{
	pnlDetailView->Visible = lvWaves->Selected;
	if (Selected == false)
		return;
	int id = lvWaves->Selected->Index;
	struct Signal::S_WAVE wave = signal[CurrentChannel()].shape[id];
	cbWaveType->ItemIndex = wave.shape;
	edAmplitude->Text = AnsiString::FloatToStrF(wave.fAmplitude, AnsiString::sffGeneral, 2, 3);
	edFrequency->Text = AnsiString::FloatToStrF(wave.fFrequency, AnsiString::sffGeneral, 2, 3);
	edPhase->Text = AnsiString::FloatToStrF(wave.fPhase, AnsiString::sffGeneral, 2, 3);
}
//---------------------------------------------------------------------------

void __fastcall TfrmSignalConfig::btnApplyClick(TObject *Sender)
{
	if (!lvWaves->Selected)
	{
		ShowMessage("No item selected");
		return;
	}
	struct Signal::S_WAVE &wave = signal[CurrentChannel()].shape[lvWaves->Selected->Index];
	float fAmpl, fFreq, fPhase;
	try
	{
		fAmpl = StrToFloat(edAmplitude->Text);
		fFreq = StrToFloat(edFrequency->Text);
		fPhase = StrToFloat(edPhase->Text);
	}
	catch (EConvertError &E)
	{
		ShowMessage("Invalid value");
		return;
	}
	if (fAmpl < 0 || fFreq < 0)
	{
		ShowMessage("Invalid value");
		return;
	}

	wave.shape = (Signal::E_WAVEFORM)cbWaveType->ItemIndex;
	wave.fAmplitude = fAmpl;
	wave.fFrequency = fFreq;
	wave.fPhase = fPhase;
	lvWaves->Invalidate();
}
//---------------------------------------------------------------------------

void __fastcall TfrmSignalConfig::cbChannelChange(TObject *Sender)
{
	this->lvWaves->Items->Count = signal[CurrentChannel()].shape.size();
	this->lvWaves->Invalidate();

	if (lvWaves->Selected)
	{
		struct Signal::S_WAVE &wave = signal[CurrentChannel()].shape[lvWaves->Selected->Index];

		cbWaveType->ItemIndex = wave.shape;
		edAmplitude->Text = AnsiString::FloatToStrF(wave.fAmplitude,  AnsiString::sffGeneral, 1, 3);
		edFrequency->Text = AnsiString::FloatToStrF(wave.fFrequency,  AnsiString::sffGeneral, 1, 3);
		edPhase->Text = AnsiString::FloatToStrF(wave.fPhase, AnsiString::sffGeneral, 1, 3);
	}
}
//---------------------------------------------------------------------------

